
function [Designs, Losses] = eg2(N, nu, delta0, n, criterion, symm, fit)

format short g
close ALL
tstart = tic;

% n is size of desired exact design
% criterion = 'I' or 'D'

% eg: [Designs, Losses] = eg2(40, 1/11, .02, 20, 'I', 'F', 'cubic');

%% Housekeeping:

if  strcmp(fit, 'linear')
    x = linspace(-1, 1, N)';
    F = [ones(N,1), x];
    [Q,~] = qr(F,0);
elseif strcmp(fit, 'quadratic')
    x = linspace(-1, 1, N)';
    F = [ones(N,1), x, x.^2];
    [Q,~] = qr(F,0);
elseif strcmp(fit, 'cubic')
    x = linspace(-1, 1, N)';
    F = [ones(N,1), x, x.^2, x.^3];
    [Q,~] = qr(F,0);
end
n0 = size(F,2);
Designs = zeros(N,5);
Designs(:,1) = x;
losses = zeros(4,1);

[beta, Lambda, ~, ~, ~, ~, xi0, xi0exact, loss, lossexact] = ...
    parametric_poly(criterion, fit, symm, N, n, nu);

Designs(:,2) = xi0;
Designs(:,3) = xi0exact;

losses(1) = loss; %Loss(Xi0)
losses(2) = lossexact; %Loss(Xi0exact)

close(1)

figure(1)
subplot(2,2,1)
barwidth = .01;
if strcmp(criterion, 'I')
    str = ['$\mathcal{I}_{\nu}(\xi^{0})$ = ', num2str(loss)];
elseif strcmp(criterion, 'D')
    str = ['$\mathcal{D}_{\nu}(\xi^{0})$ = ', num2str(loss)];
end
hold on
xlim([-1.05 1.05]);
ylim([0, 1.3*max(xi0)]);
bar(x, xi0, barwidth);
xlabel('(a)')
ylabel('\xi^{0}(x)')
textheight = 1.1*max(xi0);
text(-.75,textheight, str,'Interpreter','latex')
hold off

subplot(2,2,2)
barwidth = .01;
if strcmp(criterion, 'I')
    str = ['$\mathcal{I}_{\nu}(\xi^{0}_{(',num2str(n),')})$ = ', num2str(lossexact)];
elseif strcmp(criterion, 'D')
    str = ['$\mathcal{D}_{\nu}(\xi^{0}_{(',num2str(n),')})$ = ', num2str(lossexact)];
end
hold on
xlim([-1.05 1.05]);
ylim([0, 1.3*max(xi0)]);
bar(x, xi0exact, barwidth);
xlabel('(b)')
ylabel(['\xi^{0}_{(',num2str(n),')}(x)'])
textheight = 1.1*max(xi0);
text(-.75,textheight, str,'Interpreter','latex')
hold off

% Initial design randomly chosen: size n0
xi = zeros(N,1);
xi(randsample(N,n0)) = 1;
xi = xi/sum(xi);
[~, Inext, ~]  = summary(xi, nu, Q, criterion); % current loss and next design point
designsize = n0;
delta = 1;
Loss = [];
Istar = [];

%% Do the iterations:
while delta > delta0 || designsize < 300
    istar = Inext;
    xi = (designsize*xi + [zeros((istar-1),1); 1; zeros(N-istar, 1)])/(designsize + 1);
    designsize = designsize + 1;
    [Lnow, Inext, delta]  = summary(xi, nu, Q, criterion);
    Istar(designsize-n0) = istar;
    Loss(designsize-n0) = Lnow;
end

% Delete the initial sample; 
% form the final design xistar from Istar alone

%nfinal = designsize - n0;

% Delete the first 100 points
Istar = Istar(101:length(Istar));
nfinal = length(Istar);
%length(Istar)
%pause

tab = tabulate(Istar);
xistar = tab(:,2)/nfinal;
%[xistarloss, ~]  = summary(xistar, nu, Q, criterion);

% Symmetrize, and delete the tiniest design weights
xistarsym = (xistar + flip(xistar))/2;
xistarsym(xistarsym < .5/n) = 0; % remove small weights, then renormalize
xistarsym = xistarsym/sum(xistarsym);
[xistarsymloss, ~]  = summary(xistarsym, nu, Q, criterion);

% Do the rounding, obtain the exact design of size n
indices = (1:N)';
[supportindices, frequency, ~] = allocate(n, xistarsym, indices);
xistarexact = zeros(N,1);
xistarexact(supportindices) = frequency/n;
[xistarexactloss, ~]  = summary(xistarexact, nu, Q, criterion);

telapsed = toc(tstart);

%% Plotting

subplot(2,2,3)
if criterion == 'I'
    str = ['$\mathcal{I}_{\nu}(\xi_{sym}^{*})$ = ', num2str(xistarsymloss)];
elseif criterion == 'D'
    str = ['$\mathcal{D}_{\nu}(\xi_{sym}^{*})$ = ', num2str(xistarsymloss)];
end
hold on
xlim([-1.05 1.05]);
ylim([0, 1.3*max(xistar)]);
bar(x, xistarsym, barwidth);
xlabel('(c)')
ylabel('\xi_{sym}^{*}(x)')
textheight = 1.1*max(xistar);
text(-.75,textheight, str,'Interpreter','latex')
hold off

subplot(2,2,4)
if criterion == 'I'
    str = ['$\mathcal{I}_{\nu}(\xi_{(',num2str(n),')}^{*})$ = ', num2str(xistarexactloss)];
elseif criterion == 'D'
    str = ['$\mathcal{D}_{\nu}(\xi_{(',num2str(n),')}^{*})$ = ', num2str(xistarexactloss)];
end
hold on
xlim([-1.05 1.05]);
ylim([0, 1.3*max(xistar)]);
bar(x, xistarexact, barwidth);
xlabel('(d)')
ylabel(['\xi_{(',num2str(n),')}^{*}(x)'])
textheight = 1.1*max(xistar);
text(-.75,textheight, str,'Interpreter','latex')
hold off

%% Final output:   

disp('Parametric design:');
disp(['beta = ', num2str(beta')]);
disp('Lambda = ');
disp(Lambda);
disp('  ');

Designs(:,4) = xistar;
Designs(:,5) = xistarexact;

losses(3) = xistarsymloss; %Loss(Xistarsym)
losses(4) = xistarexactloss; %Loss(Xistarexact)
Losses = losses;
printmat(Designs, 'Designs', num2str(1:N), 'x Xi0 Xi0exact Xistarsym Xistarexact');
disp('  ');
printmat(Losses, 'Losses', 'Xi0 Xi0exact Xistarsym Xistarexact', 'Loss');
disp('  ');


%% Test the necessary condition

if nu < 1
    [~, ~, delta1]  = summary(xistar, nu, Q, criterion);
    [~, ~, delta2]  = summary(xistarsym, nu, Q, criterion);
    disp(['Final design size = ', num2str(designsize - n0), ...
        ' delta for xistar = ', num2str(delta1), ...
        ' delta for xistarsym = ', num2str(delta2)]);
else
    disp(['nstar = ', num2str(length(Istar))]);
end
telapsed
%save poly




