
function [Designs, Losses, n_seq, n_adapt]...
    = eg4(n0, delta0, N, nu, criterion)

format short g
close ALL
tstart = tic;

% n0 is size of initial design
% finaldessize is size of final design
% N is size of design space
% criterion = 'I' or 'D'

% eg: [Designs, Losses, n_seq, n_adapt] =  eg4(10, .01, 100, .5, 'I');

%% Housekeeping:
%n0 = 10;
%N = 100, 
%criterion = 'I';

X = linspace(1,100,N)';
sigma = .1;
tau = sqrt(nu/(1-nu))*sigma;
thetastar = [200; .10];
theta0 = truetheta(thetastar, X);


% initial design:
freq = zeros(N,1);
freq(randi(N,n0,1))= 1;
n0 = sum(freq);
xi_init = freq/n0;
p = length(thetastar);
Xsample = repX(X,freq);



%% Compute the parametric design
[~, Q, ~] = decomp(thetastar, X, 1); 
[alpha, beta, Lambda, xi0, loss0] = parametric(criterion, Q, nu);
alpha
beta
Lambda


%% Compute the sequential design
Istar = [];
xi = xi_init;
[~, Inext, ~, ~]  = eg4summary(xi, nu, Q, criterion); % current loss and next design point
designsize = n0;
delta = 1;

%% Do the iterations:
while delta > delta0 
    istar = Inext;
    xi = (designsize*xi + [zeros((istar-1),1); 1; zeros(N-istar, 1)])/(designsize + 1);
    designsize = designsize + 1;
    [~, Inext, delta, ~]  = eg4summary(xi, nu, Q, criterion);
    if rem(designsize, 100) == 0
        disp(['design size = ', num2str(designsize), ' delta = ', num2str(delta)]);
    end
    Istar(designsize-n0) = istar;   
end

% Delete the initial sample; 
% form the final design xistar from Istar alone
Istar1 = Istar;
tab1 = tabulate(Istar1);
n_seq = length(Istar1);
xi_seq = tab1(:,2)/n_seq;
[xi_seq_loss, ~, ~, ~]  = eg4summary(xi_seq, nu, Q, criterion);

%% Compute the adaptive design
Istar = [];
delta = 1;
n0 = size(Xsample,1);
n = n0;
z = max(eta(thetastar,Xsample) + sigma*randn(n,1),0);
theta = GaussNewton(z, Xsample);
theta = max(theta,0);
xi = xi_init;
% loop:
while delta > delta0
    [~, Q, Qcomp] = decomp(theta, X, 1);
    [~, Inext, delta, Rinv]  = eg4summary(xi, nu, Q, criterion);
    istar = Inext;
    freq(Inext) = freq(Inext) + 1;
    n = sum(freq);
    xi = freq/n;
    Xsample = repX(X,freq);
        % Compute and add in psi at this point:
        m = Rinv*Q'*diag(xi)*Qcomp;
        mat = m'*m + eye(N-p);
        [c, ~] = eigs(mat,1);
        psi = (tau/sqrt(n))*Qcomp*c;
    %psi(Inext)
    znew = max(eta(thetastar,X(Inext)) + psi(Inext) + sigma*randn,0);
    z = [z; znew];
    theta = GaussNewton(z, Xsample);
    Istar(n-n0) = istar; 
    if rem(n, 100) == 0
        disp(['design size = ', num2str(n), ' delta = ', num2str(delta)]);
    end
end

Istar2 = Istar;
tab2 = tabulate(Istar2);
n_adapt = length(Istar2);
xi_adapt = tab2(:,2)/n_adapt;

% final loss relative to the same Q as for the sequential design:
[~, Q, ~] = decomp(thetastar, X, 1);
[xi_adapt_loss, ~, ~, ~]  = eg4summary(xi_adapt, nu, Q, criterion);

func2 = @(theta)norm(z - eta(theta, Xsample))^2;
check = fminunc(func2,thetastar); %obtain the final estimate in another way

disp('  ');
disp(['nu = ', num2str(nu)]);
disp('theta*, true theta if nu = 0, final estimate and final minimizer are')
disp(round([thetastar theta0 theta check],4));

%% Plotting

Designs = [xi0 xi_seq xi_adapt];
Losses = [loss0; xi_seq_loss; xi_adapt_loss];
maxes = [max(xi0), max(xi_seq),max(xi_adapt)];

roundLosses = round(Losses, 3);

figure(1)

barwidth = .1;
textheight = .9*max(maxes);

subplot(1,3,1)
if strcmp(criterion, 'I')
    str1 = {['$\mathcal{I}_{\nu}(\xi_{0})$ = ', num2str(roundLosses(1))]};
elseif strcmp(criterion', 'D')
    str1 = {['$\mathcal{D}_{\nu}(\xi_{0})$ = ', num2str(roundLosses(1))]};
end
hold on
xlim([0,max(X) + 1]);
ylim([0 1.2*textheight]);
bar(X, xi0, barwidth);
xlabel('(a)')
ylabel('\xi(x)')
text(.3*max(X),textheight, str1,'Interpreter','latex')
hold off


subplot(1,3,2)
if strcmp(criterion, 'I')
    str1 = {['$\mathcal{I}_{\nu}(\xi_{seq})$ = ', num2str(roundLosses(2))], ['des. size = ', num2str(n_seq)]};
elseif strcmp(criterion', 'D')
    str1 = {['$\mathcal{D}_{\nu}(\xi_{seq})$ = ', num2str(roundLosses(2))], ['des. size = ', num2str(n_seq)]};
end
hold on
xlim([0,max(X) + 1]);
ylim([0 1.2*textheight]);
bar(X, xi_seq, barwidth);
xlabel('(b)')
ylabel('\xi(x)')
text(.3*max(X),textheight, str1,'Interpreter','latex')
hold off

subplot(1,3,3);
if strcmp(criterion, 'I')
    str1 = {['$\mathcal{I}_{\nu}(\xi_{adapt})$ = ', num2str(roundLosses(3))], ['des. size = ', num2str(n_adapt)]};
elseif strcmp(criterion, 'D')
    str1 = {['$\mathcal{D}_{\nu}(\xi_{adapt})$ = ',num2str(roundLosses(3))], ['des. size = ', num2str(n_adapt)]};
end
hold on
xlim([0,max(X) + 1]);
ylim([0 1.2*textheight]);
bar(X, xi, barwidth);
xlabel('(c)')
ylabel('\xi(x)')
text(.3*max(X),textheight, str1,'Interpreter','latex')
hold off

telapsed = toc(tstart)
save adaptive
 

