function [beta, Lambda, Xi0, Xi0sym, Xi0exact, Xi0exactsym, xi0, xi0exact, loss, lossexact] = ...
    parametric_poly(criterion, fit, symm, N, n, nu)

format short g
close ALL
tstart = tic;

% n is size of desired exact design
% criterion = 'I' or 'D'
% fit is fitted model

% eg: [beta, Lambda, Xi0, Xi0sym, Xi0exact, Xi0exactsym, xi0, xi0exact, loss, lossexact] = parametric_poly('I', 'linear', 'F', 40, 20, .5)

%% Housekeeping:

if  strcmp(fit, 'linear')
    x = linspace(-1, 1, N)';
    F = [ones(N,1), x];
    [Q,~] = qr(F,0);
    p = size(F,2);
    beta0 = [1;0];
    Lambda0 = rand*ones(p);
elseif strcmp(fit, 'quadratic')
    x = linspace(-1, 1, N)';
    F = [ones(N,1), x, x.^2];
    [Q,~] = qr(F,0);
    p = size(F,2);
    beta0 = [1; 0; rand]; % rand(p,1);
    Lambda0 = eye(p);
elseif strcmp(fit, 'cubic')
    x = linspace(-1, 1, N)';
    F = [ones(N,1), x, x.^2, x.^3];
    [Q,~] = qr(F,0);
    p = size(F,2);
    if strcmp(criterion, 'I') && nu > 0
        beta0 = [.91 0 .42 0]';
        Lambda0 = [ -0.86    0   -.06   0; ...
            0     .03   0   .17; ...
            -.06     0    .24    0; ...
            0    .17     0  .35];
    elseif strcmp(criterion, 'I') && nu == 0
        beta0 = [.6314 0 .7755 0]';
        Lambda0 = [-0.25         0    0.0128         0; ...
            0   -0.3227         0   -0.0751; ...
            0.0128         0   -0.2452         0; ...
            0   -0.0751         0    0.4667];
        %beta0 = beta0 + .1*randn(p,1);
        %Lambda0 = Lambda0 + .1*randn(p);
    elseif strcmp(criterion, 'D') && nu > 0
        beta0 = [0.8439           0      0.5366           0]';
        Lambda0 = [-0.8026            0       -0.044            0; ...
                        0      -0.5475            0      -0.0591; ...
                   -0.044            0      -0.4601            0; ...
                        0      -0.0591            0       0.9634];
        beta0 = beta0 + .1*randn(p,1);
        %Lambda0 = Lambda0 + .1*randn(p);
    elseif strcmp(criterion, 'D') && nu == 0
        beta0 = [.6314 0 .7755 0]';
        Lambda0 = [-21.67      0  1.1095      0; ...
            0      -27.972            0      -6.5097; ...
            1.1095      0      -21.254         0; ...
              0      -6.5097        0       40.454];
        beta0 = beta0 + .1*randn(p,1);
        %Lambda0 = Lambda0 + .1*randn(p);
    end
end
theta0 = [beta0; vecs(Lambda0, p)];
[theta0, ~, ~] = phi(theta0, Q, criterion, nu, p, symm);

fun = @(theta) lossonly(theta, Q, criterion, nu, p, symm);
nonlcon = @(theta) phicon(theta, Q, p, symm);

options = optimset('Display', 'final', 'algorithm', 'interior-point',...
    'TolFun', 1e-8, 'TolX', 1e-8, 'TolCon', 1e-8, ...
    'MaxIter', 1000, 'MaxFunEvals', 10000);
% Display 'iter' or 'final'
% try 'interior-point' (default), then 'sqp', then'active-set' algorithm
[theta, ~, exitflag, output] = ...
    fmincon(fun,theta0,[],[],[],[],[],[],nonlcon, options);
exitflag
% output

% prepare plots and final output

[theta, xi0, loss] = phi(theta, Q, criterion, nu, p, symm);
Xi0 = [x(xi0>0) xi0(xi0>0)];

% symmetrize:
xi0sym = (xi0 + flipud(xi0))/2;
[losssym, ~, ~] = summary(xi0sym, nu, Q, criterion);
Xi0sym = [x(xi0sym>0) xi0sym(xi0sym>0)];

indices = (1:N)';
[supportindices, frequency, ~] = allocate(n, xi0, indices);
Xi0exact = [x(supportindices) frequency/n];
xi0exact = zeros(N,1);
xi0exact(supportindices) = frequency/n;
[lossexact, ~, ~] = summary(xi0exact, nu, Q, criterion);

[supportindices, frequency, ~] = allocate(n, xi0sym, indices);
Xi0exactsym = [x(supportindices) frequency/n];
xi0exactsym = zeros(N,1);
xi0exactsym(supportindices) = frequency/n;
[lossexactsym, ~, ~] = summary(xi0exactsym, nu, Q, criterion);

figure(1)
subplot(2,2,1)
barwidth = .01;
if strcmp(criterion, 'I')
    str = ['$\mathcal{I}_{\nu}(\xi^{0})$ = ', num2str(loss)];
elseif strcmp(criterion, 'D')
    str = ['$\mathcal{D}_{\nu}(\xi^{0})$ = ', num2str(loss)];
end
hold on
xlim([-1.05 1.05]);
ylim([0, 1.3*max(xi0)]);
bar(x, xi0, barwidth);
xlabel('(a)')
ylabel('\xi^{0}(x)')
textheight = 1.1*max(xi0);
text(-.75,textheight, str,'Interpreter','latex')
hold off

subplot(2,2,2)
barwidth = .01;
if strcmp(criterion, 'I')
    str = ['$\mathcal{I}_{\nu}(\xi_{sym}^{0})$ = ', num2str(losssym)];
elseif strcmp(criterion, 'D')
    str = ['$\mathcal{D}_{\nu}(\xi_{sym}^{0})$ = ', num2str(losssym)];
end
hold on
xlim([-1.05 1.05]);
ylim([0, 1.3*max(xi0)]);
bar(x, xi0sym, barwidth);
xlabel('(b)')
ylabel('\xi_{sym}^{0}(x)')
textheight = 1.1*max(xi0);
text(-.75,textheight, str,'Interpreter','latex')
hold off

subplot(2,2,3)
barwidth = .01;
if strcmp(criterion, 'I')
    str = ['$\mathcal{I}_{\nu}(\xi^{0}_{exact})$ = ', num2str(lossexact)];
elseif strcmp(criterion, 'D')
    str = ['$\mathcal{D}_{\nu}(\xi^{0}_{exact})$ = ', num2str(lossexact)];
end
hold on
xlim([-1.05 1.05]);
ylim([0, 1.3*max(xi0)]);
bar(x, xi0exact, barwidth);
xlabel('(c)')
ylabel('\xi^{0}_{exact}(x)')
textheight = 1.1*max(xi0);
text(-.75,textheight, str,'Interpreter','latex')
hold off

subplot(2,2,4)
barwidth = .01;
if strcmp(criterion, 'I')
    str = ['$\mathcal{I}_{\nu}(\xi^{0}_{sym,exact})$ = ', num2str(lossexactsym)];
elseif strcmp(criterion, 'D')
    str = ['$\mathcal{D}_{\nu}(\xi^{0}_{sym,exact})$ = ', num2str(lossexactsym)];
end
hold on
xlim([-1.05 1.05]);
ylim([0, 1.3*max(xi0)]);
bar(x, xi0exactsym, barwidth);
xlabel('(d)')
ylabel('\xi^{0}_{sym,exact}(x)')
textheight = 1.1*max(xi0);
text(-.75,textheight, str,'Interpreter','latex')
hold off

rounder = 4;
beta = round(theta(1:p), rounder);
vec = theta((p+1):length(theta));
Lambda = round(unvecs(vec,p), rounder);
telapsed = toc(tstart);

%% ============================= subfunction "phi" =============================
% accepts a vector of parameters, returns (i) parameters meeting the
% constraints, (ii) design, (iii) loss, both evaluated at the new
% parameters
function [newtheta, xi, loss] = phi(theta, Q, criterion, nu, p, symm)
beta = theta(1:p);
vec = theta((p+1):length(theta));
Lambda = unvecs(vec,p);
if strcmp(symm, 'T')
    E = diag((-1).^(0:(p-1)));
    beta = (beta + E*beta)/2;
    Lambda = (Lambda + E*Lambda*E')/2;
end
xi = max(diag(Q*Lambda*Q'),0)./diag(Q*(beta*beta')*Q');

if all(xi==0)
    loss = 1e6;
    newtheta = theta + rand*ones(length(theta),1);
    N = length(xi);
    xi = ones(N,1)/N;
else    
    I = sum(xi);
    beta = beta*sqrt(I); % now sum(xi) = 1 
    Lambda = Lambda/(norm(beta)^2);
    beta = beta/norm(beta); % now norm(beta) = 1
    xi = max(diag(Q*Lambda*Q'),0)./diag(Q*(beta*beta')*Q');
    newtheta = [beta; vecs(Lambda, p)];
    [loss, ~, ~] = summary(xi, nu, Q, criterion);
end



%% ============================= subfunction "lossonly" =============================
function Loss = lossonly(theta, Q, criterion, nu, p, symm)
[~,~, Loss] = phi(theta, Q, criterion, nu, p, symm);

%% ============================= subfunction "phicon" =============================
function [c,ceq] = phicon(theta, Q, p, symm)
c = [];
beta = theta(1:p);
vec = theta((p+1):length(theta));
Lambda = unvecs(vec,p);
xi = max(diag(Q*Lambda*Q'),0)./diag(Q*(beta*beta')*Q');
if strcmp(symm, 'T')    
    E = diag((-1).^(0:(p-1)));
    ceq = [sum(xi) - 1; norm(beta) - 1; ...
        norm(beta - E*beta); norm(Lambda - E*Lambda*E')];
else
    ceq = [sum(xi) - 1; norm(beta) - 1];
end             
                
%% ============================= subfunction "vecs" =============================
%Forms a vector from the lower triangle of a matrix
function vec = vecs(mat, p)
vec = [];
for j = 1:p
	vec = [vec; mat(j:p,j)];
end
        
% ============================= subfunction "unvecs" =============================
%Forms a matrix from the output of vecs
function mat = unvecs(vec, p)
mat = zeros(p);
    for j = 1:p
        last = (j-1)*(2*p - j + 2)/2;
        mat(j:p,j) = vec((last + 1):(last + (p - j + 1)));
    end
mat = mat + mat';
mat = mat - diag(diag(mat))/2;
                     

                        
